package appdata;
use klinstall;

my $APPID = 1103;

############################## text strings ##################################

my $ASK_KLNAGENT_SERVER = <<ASK_KLNAGENT_SERVER_END;
Please enter Administration Server DNS-name or static IP-address
ASK_KLNAGENT_SERVER_END

my $ASK_KLNAGENT_PORT = <<ASK_KLNAGENT_PORT_END;
Please enter Administration Server port number
ASK_KLNAGENT_PORT_END

my $ASK_KLNAGENT_SSLPORT = <<ASK_KLNAGENT_SSLPORT_END;
Please enter Administration Server ssl port number
ASK_KLNAGENT_SSLPORT_END

my $ASK_KLNAGENT_USESSL = <<ASK_KLNAGENT_USESSL_END;
Please enter 'Y' to confirm that you want to use SSL encryption or enter 'N' to use non-encrypted communication
ASK_KLNAGENT_USESSL_END

my $ASK_KLNAGENT_EULA = <<ASK_KLNAGENT_EULA_END;
Please enter 'Y' to confirm that you accept the End User License Agreement. You must accept the terms and conditions of the End User License Agreement to install the application. Enter 'N' providing you do not accept the End User License Agreement or 'R' to show it again
ASK_KLNAGENT_EULA_END

my $ASK_KLNAGENT_GW_MODE = <<EOT;
The Network Agent being installed can be used as connection gateway. It will allow you to connect to Administration Server using the specified connection gateway.

1) Do not configure connection gateway
2) Do not use connection gateway
3) Connect to server using connection gateway
4) Use as connection gateway
Please choose connection gateway mode:
EOT

my $ASK_KLNAGENT_GW_ADDRESS = <<EOT;

Please enter connection gateway address
EOT

my $INSTALL_MSG_REGSERVER_FAIL = <<EOT;
Network agent registration failed with code '%1': '%2'
EOT

my $INSTALL_MSG_UPGRADE_FAIL = <<EOT;
Network agent upgrade failed with code '%1': '%2'
EOT

# Returns application information.
#
# Should define at least these hash values:
#
sub get_application_info
{
    my %AI;

    # Take install data
    seek (DATA, 0, 0);
    foreach (<DATA>)
    {
        chomp;
        m/^f\s+(.*)$/ and $AI{"FILES"} .= "$1\n"
            or m/^d\s+(.*)$/ and $AI{"DIRS"} .= "$1\n"
                or m/^(\w+)=(.*)$/ and $AI{$1} = $2;
    }

    klinstall::fatal ("Corrupted installation!") 
        if !$AI{"INSTROOT"};

    my $PRODUCT = 'klnagent64';

    $AI{"ID"} = $APPID;
    $AI{"PRODUCT"} = $PRODUCT;
    $AI{"NAME"} = "Kaspersky Network Agent";
    $AI{"DEFAULT_CONFIG"} = "$AI{CFGPATH}";
    $AI{"REQUIRE_AV_BASES"} = "0";
    $AI{"REQUIRE_SETUP_KEEPUP2DATE"} = "0";
    $AI{"REQUIRE_LICENSES"} = "0";
	$AI{"ARCH"} = "AMD64";
	

    $AI{"KLNAGENT_BIN"} = "$AI{INSTROOT}/sbin/klnagent";

    $AI{"COMP_SETUP_SCRIPT"} = "$AI{LIBEXEC}/setup/klnagent_setup.pl";
    $AI{"COMP_POSTINSTALL_SCRIPT"} = "$AI{LIBEXEC}/setup/postinstall.pl";

    return %AI;
}

# Generic checks
sub app_check_os_ver { return 0; }
sub app_check_linux_distro { return 0; }
sub app_get_conflict_ids { return; }
sub app_check_required_components { return; }

sub app_check_optional_components
{
}

{
    sub is_using_systemd_
    {
        my $init_process_name = `ps -o comm= -p 1`;
        chomp $init_process_name;
        return $init_process_name eq 'systemd';
    }

    my $is_systemd = is_using_systemd_();
    sub is_using_systemd
    {
        return $is_systemd;
    }
}

sub app_control_systemd
{
    my $control_action = shift;
    my %AI = appdata::get_application_info();
    my $service = "$AI{'PKGPRODUCT'}";
    my $cmd = "systemctl $control_action $service";

    return klinstall::exec_cmd ($cmd);
}

sub app_control_sysv
{
    my $control_action = shift;
    my %AI = appdata::get_application_info();
    my $cmd = "$AI{'RCCMD'} $control_action";

    return klinstall::exec_cmd ($cmd);
}

sub app_control
{
    my $control_action = shift;
    if (is_using_systemd()) {
        if ($control_action eq "start") {
            # workaround TT 2542238
            app_control_systemd "stop";
        }
        return app_control_systemd $control_action;
    } 

    return app_control_sysv $control_action;
}

sub app_start
{
    my %AI = appdata::get_application_info();

    print "Starting $AI{PRODUCT}: ";
    my ($rc, $outmsg) = app_control "start";
    print "$outmsg\n";

    return $rc == 0;
}

sub app_stop
{
    my %AI = appdata::get_application_info();

    print "Stopping $AI{PRODUCT}: ";
    my ($rc, $outmsg) = app_control "stop";
    print "$outmsg\n";

    return $rc == 0;
}

sub app_is_running
{
    if (is_using_systemd()) {
        my ($rc, $outmsg) = app_control "show -p SubState";
        return (($rc >> 8) == 0 and index($outmsg, "running") != -1)
    }

    my ($rc, $outmsg) = app_control "status";
    return ($rc >>8) == 0;
}

sub stop_deprecated_klnagent
{
    if (!is_using_systemd()) {
        return 1;
    }
    
    my $service = "klnagent";
    my ($rc, $outmsg) = klinstall::exec_cmd ("systemctl show -p SubState $service");
    
    if (($rc >> 8) == 0 and index($outmsg, "running") != -1) {
        print "Stopping $service: ";
        ($rc, $outmsg) = klinstall::exec_cmd ("systemctl stop $service");
        print "$outmsg\n";

        return $rc == 0;
    }
}

sub app_checks_preuninstall { return; }

sub app_is_license_needed { return 0; }

sub install_licenses { return; }

sub app_modify_config_file {}

sub setup_http_proxy {}

sub update_AV_database { return 1; }

sub _is_valid_ip_address
{
    my $value = shift;

    return 1 if($value =~ /^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(\/[\w\s].*)*$/ || $value =~ /^([0-9a-fA-F:])*$/);
	
    return 0;
}

sub _is_valid_host_name
{
    my $value = shift;

    return 1 if $value =~ /^(([a-zA-Z]|[a-zA-Z][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z]|[A-Za-z][A-Za-z0-9\-]*[A-Za-z0-9])(\/[\w\s].*)*$/;

    return 0;
}

sub validate_hostname
{
    my $hostname = shift;
    if (_is_valid_host_name($hostname) || _is_valid_ip_address($hostname)) {
		return 1;
    }
	
    return 0;
}

sub ask_eula
{
    my $value;
    my $show = 1;
    my %AI = appdata::get_application_info();
 
    while (1) {
        if( $show == 1 ) {
            system("more $AI{SHAREPATH}/license/license_en.txt");
            print "\n";
        }

        $value = klinstall::ask_question('KLNAGENT_EULA', "N", $ASK_KLNAGENT_EULA);

        if($value =~ /^(y(es)?|n(o)?|r(epeat)?|1|0|2)$/i) {
            if($value =~ /^(y(es)?|1)$/i) {
                last;
            } elsif($value =~ /^(n(o)?|0)$/i) {
                unlink "$AI{DATAROOT}/eula_accepted";
                klinstall::fatal ("The End User License Agreement is not accepted!");
            } else {
                $show = 1;
            }
        }
        else {
            klinstall::output("Please answer either 'Y', 'N' or 'R'.\n");
            $show = 0;
        }

        klinstall::remove_autoanswer('KLNAGENT_EULA');
    }
}

sub ask_klnagent_vars
{
    my $klnagent = ();

    my $value;

    # KLNAGENT_SERVER
    while (1) {
        $value = klinstall::ask_question ('KLNAGENT_SERVER', "", $ASK_KLNAGENT_SERVER);
		$validate_res = validate_hostname ($value);        
		last if $validate_res == 1;

        klinstall::output("Administration Server address is required and must be valid.\n");
        klinstall::remove_autoanswer('KLNAGENT_SERVER');
    }
	$klnagent->{'SERVER'} = $value;   

    # KLNAGENT_PORT
    while (1) {
        $value = klinstall::ask_question('KLNAGENT_PORT', "14000", $ASK_KLNAGENT_PORT);
        last if $value =~ /^\d+$/;

        klinstall::output("Entered port number is invalid.\n");
        klinstall::remove_autoanswer('KLNAGENT_PORT');
    }
    $klnagent->{'PORT'} = $value;

    # KLNAGENT_SSLPORT
    while (1) {
        $value = klinstall::ask_question ('KLNAGENT_SSLPORT', "13000", $ASK_KLNAGENT_SSLPORT);
        last if $value =~ /^\d+$/;

        klinstall::output("Entered port number is invalid.\n");
        klinstall::remove_autoanswer('KLNAGENT_SSLPORT');
    }
    $klnagent->{'SSLPORT'} = $value;

    # KLNAGENT_USESSL
    while (1) {
        $value = klinstall::ask_question('KLNAGENT_USESSL', "Y", $ASK_KLNAGENT_USESSL);
        last if $value =~ /^(y(es)?|n(o)?|1|0)$/i;

        klinstall::output("Please answer either 'Y' or 'N'.\n");
        klinstall::remove_autoanswer('KLNAGENT_USESSL');
    }
    $klnagent->{'USESSL'} = ($value =~ /^(y(es)?|1)$/i) ? '1' : '0';

    # KLNAGENT_GW_MODE
    while (1) {
        $value = klinstall::ask_question('KLNAGENT_GW_MODE', "1", $ASK_KLNAGENT_GW_MODE);
        last if $value =~ /^[1-4]$/;

        klinstall::output("Please answer either '1', '2', '3' or '4'.\n");
        klinstall::remove_autoanswer('KLNAGENT_GW_MODE');
    }

    $klnagent->{'GW_MODE'} = $value;

    if ($value eq '3') {
        # KLNAGENT_GW_ADDRESS
        while (1) {
            $value = klinstall::ask_question('KLNAGENT_GW_ADDRESS', "", $ASK_KLNAGENT_GW_ADDRESS);
            last if $value ne '';

            klinstall::output("Entered gateway address is invalid.\n");
            klinstall::remove_autoanswer('KLNAGENT_GW_ADDRESS');
        }

        $klnagent->{'GW_ADDRESS'} = $value;
    }

    return $klnagent;
}

sub get_tmp
{
    foreach my $dir ('/tmp', '/var/tmp') {
        return $dir if ( -w "$dir" and -d "$dir" );
    }
    return '.';
}

sub klnagent_exec_cmd_safe
{
    my @args = @_;
    my ($msg, $cmd);

    # Escape shell args
    foreach (@args)
    {
		s/([\&;\`'\\\|"*?~<>^\(\)\[\]\{\}\$\n\r])/\\$1/g;

		# if argument contains spaces
		if ($_  =~ /\s/ )
		{
			# add double quotes to the argument
			$_ = qq{"$_"};
		}
    }

    my $tmpfile = get_tmp . "/klinstallexec.$$.tmp";
    my $cmdline = join (" ", @args) . " 1>$tmpfile 2>&1";

    my $retcode = system ($cmdline);
    $retcode = ($retcode == -1 ? "undef" : $?); # -1 means system() failed

    if ( open FILE, "<$tmpfile" )
    {
        $msg = join ("", <FILE>);
        close FILE;

        unlink $tmpfile;
    }
    else 
    {
        $msg = "";
    }

    return ($retcode, $msg);
}

sub klnagent_register
{
    my $vars = shift;
    my %AI = appdata::get_application_info();

    klnagent_unregister("-beforeinstall"); # required if registered already

    my @opts;
    push @opts, $AI{"KLNAGENT_BIN"};
    push @opts, '-regserver';
    push @opts, '-pkgver',      $AI{'VER'};
    push @opts, '-unattended',  $ENV{"KLAUTOANSWERS"} if defined($ENV{"KLAUTOANSWERS"});
    push @opts, '-server',      $vars->{'SERVER'} if defined ($vars->{'SERVER'});
    push @opts, '-port',        $vars->{'PORT'} if defined ($vars->{'PORT'});
    push @opts, '-sslport',     $vars->{'SSLPORT'} if defined ($vars->{'SSLPORT'});
    push @opts, '-usessl',      $vars->{'USESSL'} if defined ($vars->{'USESSL'});
    push @opts, '-groupname',   $vars->{'GROUPNAME'} if defined($vars->{'GROUPNAME'});
    push @opts, '-gwmode',      $vars->{'GW_MODE'} if defined($vars->{'GW_MODE'});
    push @opts, '-gwaddress',   $vars->{'GW_ADDRESS'} if defined($vars->{'GW_ADDRESS'});
    push @opts, '-eula_accepted', '1';
    push @opts, '-tl', 4;
    push @opts, '-tf', "/tmp/klnagent_install.log";

    my ($rc, $msg) = klnagent_exec_cmd_safe(@opts);
    klinstall::fatal($INSTALL_MSG_REGSERVER_FAIL, $rc, $msg) unless ($rc >= 0 and $rc < 3);
}

sub klnagent_register_auto
{
    my %AI = appdata::get_application_info();

    klnagent_unregister("-beforeinstall"); # required if registered already

    my @opts;
    push @opts, $AI{"KLNAGENT_BIN"};
    push @opts, '-regserver';
    push @opts, '-pkgver',      $AI{'VER'};
    push @opts, '-unattended',  $ENV{"KLAUTOANSWERS"} if defined($ENV{"KLAUTOANSWERS"});
    push @opts, '-tl', 4;
    push @opts, '-tf', "/tmp/klnagent_install.log";

    my ($rc, $msg) = klnagent_exec_cmd_safe(@opts);
    klinstall::fatal($INSTALL_MSG_REGSERVER_FAIL, $rc, $msg) unless ($rc >= 0 and $rc < 3);
}

sub klnagent_upgrade
{
    my %AI = appdata::get_application_info();
    my ($rc, $msg) = klnagent_exec_cmd_safe ($AI{"KLNAGENT_BIN"}, '-upgrade', '-pkgver', $AI{'VER'}, '-eula_accepted', '1', '-tl', 4, '-tf', "/tmp/klnagent_upgrade.log");
    klinstall::fatal($INSTALL_MSG_UPGRADE_FAIL, $rc, $msg) unless ($rc >= 0 and $rc < 3);
}

sub klnagent_unregister
{
    my @opts = @_;
    my %AI = appdata::get_application_info();
    unshift @opts, "-unregserver";
    my ($rc, $msg) = klnagent_exec_cmd_safe ($AI{"KLNAGENT_BIN"}, @opts);
}

sub klnagent_unregister_notify
{
    klnagent_unregister('-notifyserver');
}

# interactive upgrade is a two-step process:
# 1) package manager post script                                       app_interactive_upgrade_pm
# 2) user calls postinstall script to accept eula and upgrade nagent   app_interactive_upgrade_user
sub app_interactive_upgrade_pm
{
    klinstall::output("Prepare for interactive upgrade\n");
	klnagent_unregister_notify() if app_is_running();
    stop_deprecated_klnagent();
    app_stop() if app_is_running();
}

sub app_interactive_upgrade_user
{
    klinstall::output("Start interactive upgrade\n");
    stop_deprecated_klnagent();
    app_stop() if app_is_running();

    ask_eula();

    klnagent_upgrade();
    app_start();
}

sub app_noninteractive_upgrade
{
    klinstall::output("Start automatic upgrade\n");
	klnagent_unregister_notify() if app_is_running();

    stop_deprecated_klnagent();
    app_stop() if app_is_running();

    klnagent_upgrade();
    app_start();
}

sub app_interactive_register
{
    klinstall::output("Start interactive installation\n");
    ask_eula();

	klnagent_unregister_notify() if app_is_running();
    my $klnagent_vars = ask_klnagent_vars();

    if ($klnagent_vars) {
        klnagent_register($klnagent_vars);
    }

    app_start() if not app_is_running();
}

sub app_noninteractive_register
{
    klinstall::output("Start automatic installation\n");
	klnagent_unregister_notify() if app_is_running();

    klnagent_register_auto();

    app_start() if not app_is_running();
}

sub app_unregister
{
    my $upgrade = shift;

    my %AI = appdata::get_application_info();

	klnagent_unregister_notify() if app_is_running();
	
    if ( (not $upgrade) and app_is_running() ) {
        if (!app_stop()) {
            klinstall::fatal("Can't stop application!");
        }
    }

    klnagent_unregister() if not $upgrade;
}

sub is_folder_not_empty 
{
    my $dirname = shift;
    my $result = 0;

    if ( opendir(DR, $dirname) ) 
    {
        @drList = readdir(DR);
    
        closedir(DR);

        if( grep($_ ne "." && $_ ne "..", @drList) )
        {
            $result = 1;
        }
    }

    return $result;
}

# every module should return true to be loaded successfully
1;
__DATA__
INSTROOT=/opt/kaspersky/klnagent64
VER=14.0.0.4490
CFGPATH=/etc/opt/kaspersky/klnagent
DBROOT=/var/opt/kaspersky
DATAROOT=/var/opt/kaspersky/klnagent/1103
LIBSPATH=/opt/kaspersky/klnagent64/lib
LIBEXEC=/opt/kaspersky/klnagent64/lib/bin
SHAREPATH=/opt/kaspersky/klnagent64/share
PKG_OS_VER=
PKG_OS_NAME=
RCCMD=/etc/init.d/klnagent64
PKGPRODUCT=klnagent64
LOGSPATH=/var/log/kaspersky/klnagent64
